@@ -1,7 +1,6 @@
 from __future__ import annotations
 
 import abc
-import collections as clt
 
 from typing import Any, Iterable, Mapping
 
@@ -36,16 +35,14 @@ class BaseFragmentTypesLoader:
 class DefaultFragmentTypesLoader(BaseFragmentTypesLoader):
     """Default towncrier's fragment types."""
 
-    _default_types = clt.OrderedDict(
-        [
-            # Keep in-sync with docs/tutorial.rst.
-            ("feature", {"name": "Features", "showcontent": True}),
-            ("bugfix", {"name": "Bugfixes", "showcontent": True}),
-            ("doc", {"name": "Improved Documentation", "showcontent": True}),
-            ("removal", {"name": "Deprecations and Removals", "showcontent": True}),
-            ("misc", {"name": "Misc", "showcontent": False}),
-        ]
-    )
+    _default_types = {
+        # Keep in-sync with docs/tutorial.rst.
+        "feature": {"name": "Features", "showcontent": True},
+        "bugfix": {"name": "Bugfixes", "showcontent": True},
+        "doc": {"name": "Improved Documentation", "showcontent": True},
+        "removal": {"name": "Deprecations and Removals", "showcontent": True},
+        "misc": {"name": "Misc", "showcontent": False},
+    }
 
     def load(self) -> Mapping[str, Mapping[str, Any]]:
         """Load default types."""
@@ -72,7 +69,7 @@ class ArrayFragmentTypesLoader(BaseFragmentTypesLoader):
     def load(self) -> Mapping[str, Mapping[str, Any]]:
         """Load types from toml array of mappings."""
 
-        types = clt.OrderedDict()
+        types = {}
         types_config = self.config["type"]
         for type_config in types_config:
             directory = type_config["directory"]
@@ -123,7 +120,7 @@ class TableFragmentTypesLoader(BaseFragmentTypesLoader):
             (fragment_type, self._load_options(fragment_type))
             for fragment_type in fragment_types
         ]
-        types = clt.OrderedDict(custom_types_sequence)
+        types = dict(custom_types_sequence)
         return types
 
     def _load_options(self, fragment_type: str) -> Mapping[str, Any]:
