@@ -2,25 +2,20 @@
 # See LICENSE for details.
 
 import os
+import tempfile
 
+from datetime import date
 from pathlib import Path
 from subprocess import call
 from textwrap import dedent
+from unittest.mock import patch
 
 from click.testing import CliRunner
 from twisted.trial.unittest import TestCase
 
 from .._shell import cli
 from ..build import _main
-
-
-def setup_simple_project():
-    with open("pyproject.toml", "w") as f:
-        f.write("[tool.towncrier]\n" 'package = "foo"\n')
-    os.mkdir("foo")
-    with open("foo/__init__.py", "w") as f:
-        f.write('__version__ = "1.2.3"\n')
-    os.mkdir("foo/newsfragments")
+from .helpers import read, setup_simple_project, with_isolated_runner, write
 
 
 class TestCli(TestCase):
@@ -43,6 +38,19 @@ class TestCli(TestCase):
             # newsfragment
             with open("foo/newsfragments/fix-1.2.feature", "w") as f:
                 f.write("Baz fix levitation")
+            # Towncrier supports fragments not linked to a feature
+            with open("foo/newsfragments/+anything.feature", "w") as f:
+                f.write("Orphaned feature")
+            with open("foo/newsfragments/+xxx.feature", "w") as f:
+                f.write("Another orphaned feature")
+            with open("foo/newsfragments/+123_orphaned.feature", "w") as f:
+                f.write("An orphaned feature starting with a number")
+            with open("foo/newsfragments/+12.3_orphaned.feature", "w") as f:
+                f.write("An orphaned feature starting with a dotted number")
+            with open("foo/newsfragments/+orphaned_123.feature", "w") as f:
+                f.write("An orphaned feature ending with a number")
+            with open("foo/newsfragments/+orphaned_12.3.feature", "w") as f:
+                f.write("An orphaned feature ending with a dotted number")
             # Towncrier ignores files that don't have a dot
             with open("foo/newsfragments/README", "w") as f:
                 f.write("Blah blah")
@@ -52,7 +60,7 @@ class TestCli(TestCase):
 
             result = runner.invoke(command, ["--draft", "--date", "01-01-2001"])
 
-        self.assertEqual(0, result.exit_code)
+        self.assertEqual(0, result.exit_code, result.output)
         self.assertEqual(
             result.output,
             dedent(
@@ -70,9 +78,17 @@ class TestCli(TestCase):
                 --------
 
                 - Baz levitation (baz)
-                - Baz fix levitation (#2)
+                - Baz fix levitation (fix-1.2)
                 - Adds levitation (#123)
                 - Extends levitation (#124)
+                - An orphaned feature ending with a dotted number
+                - An orphaned feature ending with a number
+                - An orphaned feature starting with a dotted number
+                - An orphaned feature starting with a number
+                - Another orphaned feature
+                - Orphaned feature
+
+
 
                 """
             ),
@@ -84,22 +100,104 @@ class TestCli(TestCase):
     def test_subcommand(self):
         self._test_command(_main)
 
-    def test_no_newsfragment_directory(self):
+    @with_isolated_runner
+    def test_in_different_dir_dir_option(self, runner):
+        """
+        The current working directory doesn't matter as long as we pass
+        the correct one.
+        """
+        project_dir = Path(".").resolve()
+
+        setup_simple_project()
+        Path("foo/newsfragments/123.feature").write_text("Adds levitation")
+        # Ensure our assetion below is meaningful.
+        self.assertFalse((project_dir / "NEWS.rst").exists())
+
+        # Create a temporary directory, run Towncrier from there and assert
+        # it didn't litter into it.
+        td = tempfile.TemporaryDirectory()
+        self.addCleanup(td.cleanup)
+
+        os.chdir(td.name)
+        result = runner.invoke(cli, ("--yes", "--dir", str(project_dir)))
+
+        self.assertEqual([], list(Path(td.name).glob("*")))
+        self.assertEqual(0, result.exit_code)
+        self.assertTrue((project_dir / "NEWS.rst").exists())
+
+    @with_isolated_runner
+    def test_in_different_dir_config_option(self, runner):
+        """
+        The current working directory and the location of the configuration
+        don't matter as long as we pass corrct paths to the directory and the
+        config file.
+        """
+        project_dir = Path(".").resolve()
+
+        setup_simple_project()
+        Path("foo/newsfragments/123.feature").write_text("Adds levitation")
+        # Ensure our assetion below is meaningful.
+        self.assertFalse((project_dir / "NEWS.rst").exists())
+
+        # Create a temporary directory, move the config file there, run
+        # Towncrier from there, and assert it didn't litter into it.
+        td = tempfile.TemporaryDirectory()
+        self.addCleanup(td.cleanup)
+
+        os.chdir(td.name)
+        (project_dir / "pyproject.toml").rename("pyproject.toml")
+        result = runner.invoke(
+            cli, ("--yes", "--config", "pyproject.toml", "--dir", str(project_dir))
+        )
+
+        # There's only pyproject.toml in this directory.
+        self.assertEqual(
+            [Path(td.name) / "pyproject.toml"], list(Path(td.name).glob("*"))
+        )
+        self.assertEqual(0, result.exit_code)
+        self.assertTrue((project_dir / "NEWS.rst").exists())
+
+    @with_isolated_runner
+    def test_in_different_dir_with_nondefault_newsfragments_directory(self, runner):
+        """
+        Using the `--dir` CLI argument, the NEWS file can
+        be generated in a sub-directory from fragments
+        that are relatives to that sub-directory.
+
+        The path passed to `--dir` becomes the
+        working directory.
+        """
+        Path("pyproject.toml").write_text(
+            "[tool.towncrier]\n" + 'directory = "changelog.d"\n'
+        )
+        Path("foo/foo").mkdir(parents=True)
+        Path("foo/foo/__init__.py").write_text("")
+        Path("foo/changelog.d").mkdir()
+        Path("foo/changelog.d/123.feature").write_text("Adds levitation")
+        self.assertFalse(Path("foo/NEWS.rst").exists())
+
+        result = runner.invoke(
+            cli,
+            ("--yes", "--config", "pyproject.toml", "--dir", "foo", "--version", "1.0"),
+        )
+
+        self.assertEqual(0, result.exit_code)
+        self.assertTrue(Path("foo/NEWS.rst").exists())
+
+    @with_isolated_runner
+    def test_no_newsfragment_directory(self, runner):
         """
         A missing newsfragment directory acts as if there are no changes.
         """
-        runner = CliRunner()
+        setup_simple_project()
+        os.rmdir("foo/newsfragments")
 
-        with runner.isolated_filesystem():
-            setup_simple_project()
-            os.rmdir("foo/newsfragments")
+        result = runner.invoke(_main, ["--draft", "--date", "01-01-2001"])
 
-            result = runner.invoke(_main, ["--draft", "--date", "01-01-2001"])
+        self.assertEqual(0, result.exit_code)
+        self.assertIn("No significant changes.\n", result.output)
 
-        self.assertEqual(1, result.exit_code, result.output)
-        self.assertIn("Failed to list the news fragment files.\n", result.output)
-
-    def test_no_newsfragments(self):
+    def test_no_newsfragments_draft(self):
         """
         An empty newsfragment directory acts as if there are no changes.
         """
@@ -113,6 +211,23 @@ class TestCli(TestCase):
         self.assertEqual(0, result.exit_code)
         self.assertIn("No significant changes.\n", result.output)
 
+    def test_no_newsfragments(self):
+        """
+        An empty newsfragment directory acts as if there are no changes and
+        removing files handles it gracefully.
+        """
+        runner = CliRunner()
+
+        with runner.isolated_filesystem():
+            setup_simple_project()
+
+            result = runner.invoke(_main, ["--date", "01-01-2001"])
+
+            news = read("NEWS.rst")
+
+        self.assertEqual(0, result.exit_code)
+        self.assertIn("No significant changes.\n", news)
+
     def test_collision(self):
         runner = CliRunner()
 
@@ -233,6 +348,8 @@ class TestCli(TestCase):
 
                   - section-b type-2 (#1)
 
+
+
             """
             ),
         )
@@ -276,10 +393,40 @@ class TestCli(TestCase):
 
                   - section-a type-1 (#1)
 
+
+
             """
             ),
         )
 
+    def test_draft_no_date(self):
+        """
+        If no date is passed, today's date is used.
+        """
+        runner = CliRunner()
+
+        with runner.isolated_filesystem():
+            setup_simple_project()
+            fragment_path1 = "foo/newsfragments/123.feature"
+            fragment_path2 = "foo/newsfragments/124.feature.rst"
+            with open(fragment_path1, "w") as f:
+                f.write("Adds levitation")
+            with open(fragment_path2, "w") as f:
+                f.write("Extends levitation")
+
+            call(["git", "init"])
+            call(["git", "config", "user.name", "user"])
+            call(["git", "config", "user.email", "user@example.com"])
+            call(["git", "config", "commit.gpgSign", "false"])
+            call(["git", "add", "."])
+            call(["git", "commit", "-m", "Initial Commit"])
+
+            today = date.today()
+            result = runner.invoke(_main, ["--draft"])
+
+            self.assertEqual(0, result.exit_code)
+            self.assertIn(f"Foo 1.2.3 ({today.isoformat()})", result.output)
+
     def test_no_confirmation(self):
         runner = CliRunner()
 
@@ -295,6 +442,7 @@ class TestCli(TestCase):
             call(["git", "init"])
             call(["git", "config", "user.name", "user"])
             call(["git", "config", "user.email", "user@example.com"])
+            call(["git", "config", "commit.gpgSign", "false"])
             call(["git", "add", "."])
             call(["git", "commit", "-m", "Initial Commit"])
 
@@ -306,6 +454,101 @@ class TestCli(TestCase):
             self.assertFalse(os.path.isfile(fragment_path1))
             self.assertFalse(os.path.isfile(fragment_path2))
 
+    @with_isolated_runner
+    def test_keep_fragments(self, runner):
+        """
+        The `--keep` option will build the full final news file
+        without deleting the fragment files and without
+        any extra CLI interaction or confirmation.
+        """
+        setup_simple_project()
+        fragment_path1 = "foo/newsfragments/123.feature"
+        fragment_path2 = "foo/newsfragments/124.feature.rst"
+        with open(fragment_path1, "w") as f:
+            f.write("Adds levitation")
+        with open(fragment_path2, "w") as f:
+            f.write("Extends levitation")
+
+        call(["git", "init"])
+        call(["git", "config", "user.name", "user"])
+        call(["git", "config", "user.email", "user@example.com"])
+        call(["git", "config", "commit.gpgSign", "false"])
+        call(["git", "add", "."])
+        call(["git", "commit", "-m", "Initial Commit"])
+
+        result = runner.invoke(_main, ["--date", "01-01-2001", "--keep"])
+
+        self.assertEqual(0, result.exit_code)
+        # The NEWS file is created.
+        # So this is not just `--draft`.
+        self.assertTrue(os.path.isfile("NEWS.rst"))
+        self.assertTrue(os.path.isfile(fragment_path1))
+        self.assertTrue(os.path.isfile(fragment_path2))
+
+    @with_isolated_runner
+    def test_yes_keep_error(self, runner):
+        """
+        It will fail to perform any action when the
+        conflicting --keep and --yes options are provided.
+
+        Called twice with the different order of --keep and --yes options
+        to make sure both orders are validated since click triggers the validator
+        in the order it parses the command line.
+        """
+        setup_simple_project()
+        fragment_path1 = "foo/newsfragments/123.feature"
+        fragment_path2 = "foo/newsfragments/124.feature.rst"
+        with open(fragment_path1, "w") as f:
+            f.write("Adds levitation")
+        with open(fragment_path2, "w") as f:
+            f.write("Extends levitation")
+
+        call(["git", "init"])
+        call(["git", "config", "user.name", "user"])
+        call(["git", "config", "user.email", "user@example.com"])
+        call(["git", "config", "commit.gpgSign", "false"])
+        call(["git", "add", "."])
+        call(["git", "commit", "-m", "Initial Commit"])
+
+        result = runner.invoke(_main, ["--date", "01-01-2001", "--yes", "--keep"])
+        self.assertEqual(1, result.exit_code)
+
+        result = runner.invoke(_main, ["--date", "01-01-2001", "--keep", "--yes"])
+        self.assertEqual(1, result.exit_code)
+
+    def test_confirmation_says_no(self):
+        """
+        If the user says "no" to removing the newsfragements, we end up with
+        a NEWS.rst AND the newsfragments.
+        """
+        runner = CliRunner()
+
+        with runner.isolated_filesystem():
+            setup_simple_project()
+            fragment_path1 = "foo/newsfragments/123.feature"
+            fragment_path2 = "foo/newsfragments/124.feature.rst"
+            with open(fragment_path1, "w") as f:
+                f.write("Adds levitation")
+            with open(fragment_path2, "w") as f:
+                f.write("Extends levitation")
+
+            call(["git", "init"])
+            call(["git", "config", "user.name", "user"])
+            call(["git", "config", "user.email", "user@example.com"])
+            call(["git", "config", "commit.gpgSign", "false"])
+            call(["git", "add", "."])
+            call(["git", "commit", "-m", "Initial Commit"])
+
+            with patch("towncrier.build.click.confirm") as m:
+                m.return_value = False
+                result = runner.invoke(_main, [])
+
+            self.assertEqual(0, result.exit_code)
+            path = "NEWS.rst"
+            self.assertTrue(os.path.isfile(path))
+            self.assertTrue(os.path.isfile(fragment_path1))
+            self.assertTrue(os.path.isfile(fragment_path2))
+
     def test_needs_config(self):
         """
         Towncrier needs a configuration file.
@@ -318,6 +561,19 @@ class TestCli(TestCase):
         self.assertEqual(1, result.exit_code, result.output)
         self.assertTrue(result.output.startswith("No configuration file found."))
 
+    @with_isolated_runner
+    def test_needs_version(self, runner: CliRunner):
+        """
+        If the configuration file doesn't specify a version or a package, the version
+        option is required.
+        """
+        write("towncrier.toml", "[tool.towncrier]")
+
+        result = runner.invoke(_main, ["--draft"], catch_exceptions=False)
+
+        self.assertEqual(2, result.exit_code)
+        self.assertIn("Error: '--version' is required", result.output)
+
     def test_projectless_changelog(self):
         """In which a directory containing news files is built into a changelog
 
@@ -370,6 +626,8 @@ class TestCli(TestCase):
             - Adds levitation (#123)
             - Extends levitation (#124)
 
+
+
             """
             ).lstrip(),
         )
@@ -410,6 +668,8 @@ class TestCli(TestCase):
 
             - Adds levitation (#123)
 
+
+
             """
             ).lstrip(),
         )
@@ -452,6 +712,8 @@ class TestCli(TestCase):
 
             - Adds levitation (#123)
 
+
+
             """
             ).lstrip(),
         )
@@ -497,6 +759,8 @@ class TestCli(TestCase):
 
             - Adds levitation (#123)
 
+
+
             """
             ).lstrip(),
         )
@@ -558,10 +822,8 @@ class TestCli(TestCase):
             self.assertTrue(os.path.exists("7.9.0-notes.rst"), os.listdir("."))
 
             outputs = []
-            with open("7.8.9-notes.rst") as f:
-                outputs.append(f.read())
-            with open("7.9.0-notes.rst") as f:
-                outputs.append(f.read())
+            outputs.append(read("7.8.9-notes.rst"))
+            outputs.append(read("7.9.0-notes.rst"))
 
             self.assertEqual(
                 outputs[0],
@@ -636,6 +898,7 @@ class TestCli(TestCase):
                     "01-01-2001",
                     "--yes",
                 ],
+                catch_exceptions=False,
             )
             # not git repository, manually remove fragment file
             Path(f"newsfragments/{fragment_file}").unlink()
@@ -669,8 +932,7 @@ class TestCli(TestCase):
             )
             self.assertTrue(os.path.exists("{version}-notes.rst"), os.listdir("."))
 
-            with open("{version}-notes.rst") as f:
-                output = f.read()
+            output = read("{version}-notes.rst")
 
             self.assertEqual(
                 output,
@@ -700,7 +962,7 @@ class TestCli(TestCase):
         """
         When all_bullets is false, subsequent lines are not indented.
 
-        The automatic ticket number inserted by towcier will allign with the
+        The automatic ticket number inserted by towncrier will align with the
         manual bullet.
         """
         runner = CliRunner()
@@ -714,7 +976,7 @@ class TestCli(TestCase):
                 )
             os.mkdir("newsfragments")
             with open("newsfragments/123.feature", "w") as f:
-                f.write("wow!\n" "~~~~\n" "\n" "No indentation at all.")
+                f.write("wow!\n~~~~\n\nNo indentation at all.")
             with open("newsfragments/124.bugfix", "w") as f:
                 f.write("#. Numbered bullet list.")
             with open("newsfragments/125.removal", "w") as f:
@@ -736,8 +998,7 @@ class TestCli(TestCase):
             )
 
             self.assertEqual(0, result.exit_code, result.output)
-            with open("NEWS.rst") as f:
-                output = f.read()
+            output = read("NEWS.rst")
 
         self.assertEqual(
             output,
@@ -832,6 +1093,8 @@ Deprecations and Removals
             - Adds levitation (#123)
             - Extends levitation (#124)
 
+
+
         """
         )
 
@@ -890,6 +1153,7 @@ Deprecations and Removals
                     "20-01-2001",
                     "--draft",
                 ],
+                catch_exceptions=False,
             )
 
         expected_output = dedent(
@@ -931,7 +1195,7 @@ Deprecations and Removals
             with open("newsfragments/123.feature", "w") as f:
                 f.write("Adds levitation")
             with open("NEWS.rst", "w") as f:
-                f.write("a line\n\nanother\n\nRelease notes start marker\n")
+                f.write("a line\n\nanother\n\nRelease notes start marker\na footer!\n")
 
             result = runner.invoke(
                 _main,
@@ -948,8 +1212,7 @@ Deprecations and Removals
 
             self.assertEqual(0, result.exit_code, result.output)
             self.assertTrue(os.path.exists("NEWS.rst"), os.listdir("."))
-            with open("NEWS.rst") as f:
-                output = f.read()
+            output = read("NEWS.rst")
 
         expected_output = dedent(
             """\
@@ -967,11 +1230,109 @@ Deprecations and Removals
             - Adds levitation (#123)
 
 
+            a footer!
         """
         )
 
         self.assertEqual(expected_output, output)
 
+    @with_isolated_runner
+    def test_default_start_string(self, runner):
+        """
+        The default start string is ``.. towncrier release notes start``.
+        """
+        setup_simple_project()
+
+        write("foo/newsfragments/123.feature", "Adds levitation")
+        write(
+            "NEWS.rst",
+            contents="""
+                a line
+
+                another
+
+                .. towncrier release notes start
+
+                a footer!
+            """,
+            dedent=True,
+        )
+
+        result = runner.invoke(_main, ["--date", "01-01-2001"], catch_exceptions=False)
+        self.assertEqual(0, result.exit_code, result.output)
+        output = read("NEWS.rst")
+
+        expected_output = dedent(
+            """
+            a line
+
+            another
+
+            .. towncrier release notes start
+
+            Foo 1.2.3 (01-01-2001)
+            ======================
+
+            Features
+            --------
+
+            - Adds levitation (#123)
+
+
+            a footer!
+            """
+        )
+
+        self.assertEqual(expected_output, output)
+
+    @with_isolated_runner
+    def test_default_start_string_markdown(self, runner):
+        """
+        The default start string is ``<!-- towncrier release notes start -->`` for
+        Markdown.
+        """
+        setup_simple_project(extra_config='filename = "NEWS.md"')
+
+        write("foo/newsfragments/123.feature", "Adds levitation")
+        write(
+            "NEWS.md",
+            contents="""
+                a line
+
+                another
+
+                <!-- towncrier release notes start -->
+
+                a footer!
+            """,
+            dedent=True,
+        )
+
+        result = runner.invoke(_main, ["--date", "01-01-2001"], catch_exceptions=False)
+        self.assertEqual(0, result.exit_code, result.output)
+        output = read("NEWS.md")
+
+        expected_output = dedent(
+            """
+            a line
+
+            another
+
+            <!-- towncrier release notes start -->
+
+            # Foo 1.2.3 (01-01-2001)
+
+            ### Features
+
+            - Adds levitation (#123)
+
+
+            a footer!
+            """
+        )
+
+        self.assertEqual(expected_output, output)
+
     def test_with_topline_and_template_and_draft(self):
         """
         Spacing is proper when drafting with a topline and a template.
@@ -1038,6 +1399,7 @@ Deprecations and Removals
 
             - Adds levitation
 
+
         """
         )
 
