@@ -3,31 +3,30 @@
 
 import os
 import os.path
-import sys
+import warnings
 
-from subprocess import PIPE, Popen, call
+from pathlib import Path
+from subprocess import call
 
 from click.testing import CliRunner
 from twisted.trial.unittest import TestCase
 
+from towncrier import check
+from towncrier.build import _main as towncrier_build
 from towncrier.check import _main as towncrier_check
 
+from .helpers import setup_simple_project, with_isolated_runner, write
 
-def create_project(pyproject_path="pyproject.toml", main_branch="main"):
+
+def create_project(
+    pyproject_path="pyproject.toml", main_branch="main", extra_config=""
+):
     """
     Create the project files in the main branch that already has a
     news-fragment and then switch to a new in-work branch.
     """
-    with open(pyproject_path, "w") as f:
-        f.write("[tool.towncrier]\n" 'package = "foo"\n')
-    os.mkdir("foo")
-    with open("foo/__init__.py", "w") as f:
-        f.write('__version__ = "1.2.3"\n')
-    os.mkdir("foo/newsfragments")
-    fragment_path = "foo/newsfragments/123.feature"
-    with open(fragment_path, "w") as f:
-        f.write("Adds levitation")
-
+    setup_simple_project(pyproject_path=pyproject_path, extra_config=extra_config)
+    Path("foo/newsfragments/123.feature").write_text("Adds levitation")
     initial_commit(branch=main_branch)
     call(["git", "checkout", "-b", "otherbranch"])
 
@@ -42,18 +41,6 @@ def commit(message):
     call(["git", "commit", "-m", message])
 
 
-def write(path, contents):
-    """Create a file with given contents including any missing parent directories"""
-    dir = os.path.dirname(path)
-    if dir:
-        try:
-            os.makedirs(dir)
-        except OSError:  # pragma: no cover
-            pass
-    with open(path, "w") as f:
-        f.write(contents)
-
-
 def initial_commit(branch="main"):
     """
     Create a git repo, configure it and make an initial commit
@@ -118,35 +105,46 @@ class TestChecker(TestCase):
                 result.output,
             )
 
-    def test_fragment_exists(self):
-        runner = CliRunner()
+    @with_isolated_runner
+    def test_fragment_exists(self, runner):
+        create_project("pyproject.toml")
 
-        with runner.isolated_filesystem():
-            create_project("pyproject.toml", main_branch="master")
+        write("foo/somefile.py", "import os")
+        commit("add a file")
 
-            file_path = "foo/somefile.py"
-            with open(file_path, "w") as f:
-                f.write("import os")
+        fragment_path = Path("foo/newsfragments/1234.feature").absolute()
+        write(fragment_path, "Adds gravity back")
+        commit("add a newsfragment")
 
-            call(["git", "add", "foo/somefile.py"])
-            call(["git", "commit", "-m", "add a file"])
+        result = runner.invoke(towncrier_check, ["--compare-with", "main"])
 
-            fragment_path = "foo/newsfragments/1234.feature"
-            with open(fragment_path, "w") as f:
-                f.write("Adds gravity back")
+        self.assertTrue(
+            result.output.endswith("Found:\n1. " + str(fragment_path) + "\n"),
+            (result.output, str(fragment_path)),
+        )
+        self.assertEqual(0, result.exit_code, result.output)
 
-            call(["git", "add", fragment_path])
-            call(["git", "commit", "-m", "add a newsfragment"])
+    @with_isolated_runner
+    def test_fragment_exists_hidden(self, runner):
+        """
+        Location of fragments can be configured using tool.towncrier.directory.
+        """
+        create_project("pyproject.toml", extra_config="directory = 'deep/fragz'\n")
 
-            result = runner.invoke(towncrier_check, ["--compare-with", "master"])
+        write("foo/bar/somefile.py", "import os")
+        commit("add a file")
 
-            self.assertTrue(
-                result.output.endswith(
-                    "Found:\n1. " + os.path.abspath(fragment_path) + "\n"
-                ),
-                result,
-            )
-            self.assertEqual(0, result.exit_code, result)
+        fragment_path = Path("deep/fragz/1234.feature").absolute()
+        write(fragment_path, "Adds gravity back")
+        commit("add a newsfragment")
+
+        result = runner.invoke(towncrier_check, ["--compare-with", "main"])
+
+        self.assertTrue(
+            result.output.endswith("Found:\n1. " + str(fragment_path) + "\n"),
+            (result.output, str(fragment_path)),
+        )
+        self.assertEqual(0, result.exit_code, result.output)
 
     def test_fragment_missing(self):
         runner = CliRunner()
@@ -184,15 +182,11 @@ class TestChecker(TestCase):
             call(["git", "add", fragment_path])
             call(["git", "commit", "-m", "add a newsfragment"])
 
-            proc = Popen(
-                [sys.executable, "-m", "towncrier.check", "--compare-with", "master"],
-                stdout=PIPE,
-                stderr=PIPE,
-            )
-            stdout, stderr = proc.communicate()
+            runner = CliRunner(mix_stderr=False)
+            result = runner.invoke(towncrier_check, ["--compare-with", "master"])
 
-        self.assertEqual(0, proc.returncode)
-        self.assertEqual(b"", stderr)
+        self.assertEqual(0, result.exit_code)
+        self.assertEqual(0, len(result.stderr))
 
     def test_first_release(self):
         """
@@ -209,7 +203,7 @@ class TestChecker(TestCase):
             # Before any release, the NEWS file might no exist.
             self.assertNotIn("NEWS.rst", os.listdir("."))
 
-            call(["towncrier", "build", "--yes", "--version", "1.0"])
+            runner.invoke(towncrier_build, ["--yes", "--version", "1.0"])
             commit("Prepare a release")
             # When missing,
             # the news file is automatically created with a new release.
@@ -237,7 +231,7 @@ class TestChecker(TestCase):
 
             # Do a first release without any checks.
             # And merge the release branch back into the main branch.
-            call(["towncrier", "build", "--yes", "--version", "1.0"])
+            runner.invoke(towncrier_build, ["--yes", "--version", "1.0"])
             commit("First release")
             # The news file is now created.
             self.assertIn("NEWS.rst", os.listdir("."))
@@ -262,7 +256,7 @@ class TestChecker(TestCase):
 
             # We now have the new release branch.
             call(["git", "checkout", "-b", "next-release"])
-            call(["towncrier", "build", "--yes", "--version", "2.0"])
+            runner.invoke(towncrier_build, ["--yes", "--version", "2.0"])
             commit("Second release")
 
             # Act
@@ -271,3 +265,133 @@ class TestChecker(TestCase):
             # Assert
             self.assertEqual(0, result.exit_code, (result, result.output))
             self.assertIn("Checks SKIPPED: news file changes detected", result.output)
+
+    def test_get_default_compare_branch_missing(self):
+        """
+        If there's no recognized remote origin, exit with an error.
+        """
+        runner = CliRunner()
+
+        with runner.isolated_filesystem():
+            create_project()
+
+            result = runner.invoke(towncrier_check)
+
+        self.assertEqual(1, result.exit_code)
+        self.assertEqual("Could not detect default branch. Aborting.\n", result.output)
+
+    def test_get_default_compare_branch_main(self):
+        """
+        If there's a remote branch origin/main, prefer it over everything else.
+        """
+        branch = check._get_default_compare_branch(["origin/master", "origin/main"])
+
+        self.assertEqual("origin/main", branch)
+
+    def test_get_default_compare_branch_fallback(self):
+        """
+        If there's origin/master and no main, use it and warn about it.
+        """
+        with warnings.catch_warnings(record=True) as w:
+            branch = check._get_default_compare_branch(["origin/master", "origin/foo"])
+
+        self.assertEqual("origin/master", branch)
+        self.assertTrue(w[0].message.args[0].startswith('Using "origin/master'))
+
+    @with_isolated_runner
+    def test_in_different_dir_with_nondefault_newsfragments_directory(self, runner):
+        """
+        It can check the fragments located in a sub-directory
+        that is specified using the `--dir` CLI argument.
+        """
+        main_branch = "main"
+        Path("pyproject.toml").write_text(
+            # Important to customize `config.directory` because the default
+            # already supports this scenario.
+            "[tool.towncrier]\n"
+            + 'directory = "changelog.d"\n'
+        )
+        subproject1 = Path("foo")
+        (subproject1 / "foo").mkdir(parents=True)
+        (subproject1 / "foo/__init__.py").write_text("")
+        (subproject1 / "changelog.d").mkdir(parents=True)
+        (subproject1 / "changelog.d/123.feature").write_text("Adds levitation")
+        initial_commit(branch=main_branch)
+        call(["git", "checkout", "-b", "otherbranch"])
+
+        # We add a code change but forget to add a news fragment.
+        write(subproject1 / "foo/somefile.py", "import os")
+        commit("add a file")
+        result = runner.invoke(
+            towncrier_check,
+            (
+                "--config",
+                "pyproject.toml",
+                "--dir",
+                str(subproject1),
+                "--compare-with",
+                "main",
+            ),
+        )
+
+        self.assertEqual(1, result.exit_code)
+        self.assertTrue(
+            result.output.endswith("No new newsfragments found on this branch.\n")
+        )
+
+        # We add the news fragment.
+        fragment_path = (subproject1 / "changelog.d/124.feature").absolute()
+        write(fragment_path, "Adds gravity back")
+        commit("add a newsfragment")
+        result = runner.invoke(
+            towncrier_check,
+            ("--config", "pyproject.toml", "--dir", "foo", "--compare-with", "main"),
+        )
+
+        self.assertEqual(0, result.exit_code, result.output)
+        self.assertTrue(
+            result.output.endswith("Found:\n1. " + str(fragment_path) + "\n"),
+            (result.output, str(fragment_path)),
+        )
+
+        # We add a change in a different subproject without a news fragment.
+        # Checking subproject1 should pass.
+        subproject2 = Path("bar")
+        (subproject2 / "bar").mkdir(parents=True)
+        (subproject2 / "changelog.d").mkdir(parents=True)
+        write(subproject2 / "bar/somefile.py", "import os")
+        commit("add a file")
+        result = runner.invoke(
+            towncrier_check,
+            (
+                "--config",
+                "pyproject.toml",
+                "--dir",
+                subproject1,
+                "--compare-with",
+                "main",
+            ),
+        )
+
+        self.assertEqual(0, result.exit_code, result.output)
+        self.assertTrue(
+            result.output.endswith("Found:\n1. " + str(fragment_path) + "\n"),
+            (result.output, str(fragment_path)),
+        )
+
+        # Checking subproject2 should result in an error.
+        result = runner.invoke(
+            towncrier_check,
+            (
+                "--config",
+                "pyproject.toml",
+                "--dir",
+                subproject2,
+                "--compare-with",
+                "main",
+            ),
+        )
+        self.assertEqual(1, result.exit_code)
+        self.assertTrue(
+            result.output.endswith("No new newsfragments found on this branch.\n")
+        )
